/** ClientIO */
package emporer.client.io;

import emporer.domain.User;
import emporer.domain.session.Request;
import emporer.domain.session.Response;
import org.json.JSONObject;

import java.io.*;
import java.net.Socket;
import java.net.UnknownHostException;
import java.text.ParseException;
import java.util.logging.Logger;

/**
 * Holds information regarding the client's connection to the server and
 * performs common I/O actions between the client and the server.
 */
public class ClientIO {
	
	/**
	 * Field LOGGER.
	 */
	private static final Logger LOGGER = Logger.getLogger(ClientIO.class.getName());
	
	/**
	 * Field SOCKET.
	 */
	private static Socket SOCKET;

	/**
	 * Field OUT.
	 */
	private static Writer OUT;
	
	/**
	 * Field IN.
	 */
	private static Reader IN;
	
	private static final Integer PORT = 12345;

	private static final String HOST = "localhost";
	
	public static User USER;
	
	
	/**
	 * connects to the server
	 */
	public static void connect() {
		try {
			ClientIO.setSocket(new Socket(HOST, PORT));
		} catch (final UnknownHostException e) {
            LOGGER.severe("Could not connect to host " + HOST + " on port " + PORT);
		} catch (final IOException e) {
            LOGGER.severe("Could not connect to host " + HOST + " on port " + PORT + ". Connection refused!");
		}
	}
	/**
	 * Sets the connected Socket for the client.
	 * @param socket
	 */
	private static void setSocket(final Socket socket) {
		SOCKET = socket;
		try {
			//Store the writer and reader for this socket
			setWriter(new OutputStreamWriter(SOCKET.getOutputStream(), "ASCII"));
			setReader(new InputStreamReader(
					new BufferedInputStream(SOCKET.getInputStream())));
		} catch (final IOException ex) {
			LOGGER.info(ex.getMessage());
		} 
	}
	
	public static OutputStream getOutputStream() {
		try {
			return SOCKET.getOutputStream();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	public static InputStream getInputStream() {
		try {
			return SOCKET.getInputStream();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
	 * Sets the writer for the Socket.
	 * @param out
	 */
	public static void setWriter(final Writer out) {
		OUT = out;
	}
	
	/**
	 * Sets the reader for the Socket.
	 * @param in
	 */
	public static void setReader(final Reader in) {
		IN = in;
	}
	
	/**
	 * Write a given String to the server.
	 * @param s
	 */
	private static void write(String s) {
        if (OUT == null) {
            LOGGER.severe("No connection established, cannot write to socket.");
            return;
        }

		try {
            OUT.write(s + "\r");
            OUT.flush();
		} catch (final IOException ex) {
			LOGGER.warning(ex.getMessage());
		}
	}
	
	/**
	 * Read a stream from the server and parse it into a JSONObject.
	
	 * @return JSONObject */
	private static JSONObject read() {
		String read = "";

        if (IN == null) {
            LOGGER.severe("No connection established, cannot read from socket.");
            return null;
        }

		try {
			for(;;) {
				int c = IN.read();
				if (c == '\r' || c == '\n' || c == -1) break;
				read += (char) c;
			}
		} catch (final IOException ex) {
			LOGGER.warning(ex.getMessage());
		}
		
		JSONObject json = null;

		try {
			json = new JSONObject(read);
		} catch (ParseException e) {
			LOGGER.warning("Could not parse response into proper JSON.");
		}
		
		return json;
	}
	
	/**
	 * Sends a Request to the server.
	 * @param request
	 */
	public static void sendRequest(final Request request) {
		if (request.toString() != null) {
			write(request.toString());
		} else {
			LOGGER.warning("Could not send request!");
		}
	}
	
	/**
	 * Waits for a response from the server and then returns it after
	 * it comes in.
	 * @return Response 
	 * */
	public static Response getResponse() {
        JSONObject responseJson = read();

        if (responseJson == null) {
            return new Response();
        }

		return new Response(responseJson);
	}
	
	/**
	 * returns the super's toString
	
	 * @return String super's toString */
	public String toString() {
		return super.toString();
	}
}
