package emporer.domain.session;

import java.text.ParseException;
import java.util.logging.Logger;

import org.json.JSONObject;

import emporer.domain.action.Action;
import emporer.json.JSONSerializable;

/**
 * Request which will give a response
 */
public class Request implements JSONSerializable {
	
	private final Action action;
	
	private JSONObject requestJSON;
	
	private JSONObject json;
	
	private static final Logger LOGGER = Logger.getLogger(Request.class.getName());
	
	/**
	 * Constructor for Request.
	 */
	public Request() {
		action = null;
		requestJSON = new JSONObject();
	}
	
	/**
	 * Constructor for Request.
	 * @param requestStr String
	 */
	public Request(final String requestStr) {
		try {
			json = new JSONObject(requestStr);
			requestJSON = new JSONObject(requestStr);
		} catch (final ParseException ex) {
			LOGGER.warning("Parse Exception" + ex);
		}
		
		if (requestJSON != null && requestJSON.getString("action") != null) {
			action = Action.getAction(requestJSON.getString("action"));
			requestJSON.remove("action");
		} else {
			action = null;
		}
	}
	
	/**
	 * Constructor for Request.
	 * @param action Action
	 * @param request JSONObject
	 */
	public Request(final Action action, final JSONObject request) {
		this.action = action;
		this.requestJSON = request;
		try {
			json = new JSONObject(request.toString());
			json.put("action", action);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			LOGGER.info("parse exception" + e);
			e.printStackTrace();
		}
		
	}
	
	@Override
	public JSONObject getJSON() {
		return json;
	}
	
	/**
	 * Method getRequest.
	 * @return JSONObject */
	public JSONObject getRequest() {
		return requestJSON;
	}
	
	@Override
	public String toString() {
		return json.toString();
	}
	
	/**
	 * Method getAction.
	 * @return Action */
	public Action getAction() {
		return action;
	}

	@Override
	public void populateFromJSON(JSONObject json) {
		// TODO write method
		
	}
}
